"use strict";

require("core-js/modules/es.array.filter");

require("core-js/modules/es.array.includes");

require("core-js/modules/es.array.map");

require("core-js/modules/es.array.reduce");

require("core-js/modules/es.array.sort");

require("core-js/modules/es.number.constructor");

require("core-js/modules/es.number.min-safe-integer");

require("core-js/modules/es.string.includes");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sortBy = sortBy;
exports.inRange = inRange;
exports.without = without;
exports.castArray = castArray;
exports.isEmpty = isEmpty;
exports.last = last;

// Functions that we need from lodash, but implemented here so we don't have to install all of lodash
function castArray(value) {
  return Array.isArray(value) ? value : [value];
}

function without(array, values) {
  return array.filter(function (value) {
    return !castArray(values).includes(value);
  });
}

function isEmpty(value) {
  if (value == null) {
    return true;
  }

  if (Array.isArray(value) || typeof value == 'string') {
    return !value.length;
  }

  if (value && value.size != null) {
    return !value.size;
  }

  for (var key in value) {
    if (hasOwnProperty.call(value, key)) {
      return false;
    }
  }

  return true;
}

function last(array) {
  var length = array == null ? 0 : array.length;
  return length ? array[length - 1] : undefined;
}

function inRange(number, start, end) {
  if (end === undefined) {
    end = start;
    start = 0;
  }

  return number >= Math.min(start, end) && number < Math.max(start, end);
}

function sortBy(array, funcs) {
  var length = funcs.length;

  var priority = function priority(value) {
    return funcs.reduce(function (acc, func, index) {
      return Math.max(acc, func(value) ? length - index : Number.MIN_SAFE_INTEGER);
    }, 0);
  };

  return array.map(function (value) {
    return {
      value: value,
      priority: priority(value)
    };
  }).sort(function (a, b) {
    return b.priority - a.priority;
  }).map(function (item) {
    return item.value;
  });
}