"use strict";

require("core-js/modules/es.array.filter");

require("core-js/modules/es.array.for-each");

require("core-js/modules/es.array.includes");

require("core-js/modules/es.array.map");

require("core-js/modules/es.array.reduce");

require("core-js/modules/es.array.reverse");

require("core-js/modules/es.array.splice");

require("core-js/modules/es.string.ends-with");

require("core-js/modules/es.string.includes");

require("core-js/modules/es.string.starts-with");

require("core-js/modules/web.dom-collections.for-each");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _essentialLodash = require("./essentialLodash");

var _fuzzyset = _interopRequireDefault(require("fuzzyset.js"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var fuzzySearch = function fuzzySearch(item, searchTerm) {
  var fuzzy = (0, _fuzzyset.default)();
  fuzzy.add(item.label.toLowerCase());
  var result = fuzzy.get(searchTerm);
  return result ? result.length : false;
};

var filterOptions = function filterOptions(props, searchTerm) {
  var options = (0, _essentialLodash.without)(props.options, props.selection);

  if (props.hasOptionGroups) {
    // Remove child groups if parent is selected
    var groupsToRemove = props.selection.filter(function (selection) {
      return selection.parent;
    }).map(function (selection) {
      return selection.group;
    });
    options = options.filter(function (option) {
      return !groupsToRemove.includes(option.group);
    });
  }

  if ((0, _essentialLodash.isEmpty)(searchTerm)) return options;
  searchTerm = searchTerm.toLowerCase();

  var filter = function filter(item) {
    return item.label.toLowerCase().includes(searchTerm) || item.value != null && item.value.toLowerCase() === searchTerm || fuzzySearch(item, searchTerm);
  };

  options = options.filter(function (item) {
    return item.parent || filter(item);
  });

  if (props.hasOptionGroups) {
    // Remove empty groups
    options.reduce(function (acc, item, index) {
      if (item.parent && !filter(item)) acc.push(index);
      return acc;
    }, []).reverse().forEach(function (index) {
      if (options[index + 1] == null || options[index].group !== options[index + 1].group) {
        options.splice(index, 1);
      }
    });
  }

  options = (0, _essentialLodash.sortBy)(options, [function (item) {
    return item.group;
  }, function (item) {
    return item.label.toLowerCase() === searchTerm;
  }, function (item) {
    return item.value != null && item.value.toLowerCase() === searchTerm;
  }, function (item) {
    return item.label.toLowerCase().startsWith(searchTerm);
  }, function (item) {
    return item.label.toLowerCase().endsWith(searchTerm);
  }, function (item) {
    return fuzzySearch(item, searchTerm);
  }]);
  return options;
};

var _default = filterOptions;
exports.default = _default;